# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-19 23:30
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('camara_deputados', '0004_auto_20170719_2314'),
    ]

    operations = [
        migrations.AddField(
            model_name='proposal',
            name='indexes',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
        migrations.AddField(
            model_name='proposal',
            name='link',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
    ]
