# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-19 23:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('camara_deputados', '0002_auto_20170719_2250'),
    ]

    operations = [
        migrations.AlterField(
            model_name='proposal',
            name='summary',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='proposal',
            name='summary_explanation',
            field=models.TextField(blank=True, null=True),
        ),
    ]
