# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-19 22:50
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('camara_deputados', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Proposal',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('year', models.IntegerField()),
                ('number', models.IntegerField()),
                ('submission_date', models.DateField()),
                ('name', models.CharField(max_length=500)),
                ('summary', models.TextField()),
                ('summary_explanation', models.TextField()),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='camara_deputados.Deputy')),
            ],
            options={
                'verbose_name': 'Proposal',
                'verbose_name_plural': 'Proposals',
            },
        ),
        migrations.CreateModel(
            name='ProposalType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField()),
                ('description', models.CharField(max_length=500)),
                ('gender', models.CharField(max_length=2)),
                ('initials', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name': 'Proposal Type',
                'verbose_name_plural': 'Proposal Types',
            },
        ),
        migrations.AddField(
            model_name='proposal',
            name='proposal_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='camara_deputados.ProposalType'),
        ),
    ]
